#!/usr/bin/perl -w

# AnotherLevelUp: bin/filemonitor.pl
# Author John Latham, Dept. CS, University of Manchester, UK
# Email jtl@cs.man.ac.uk
# Licence: GPL
# Copyright John Latham 2000, 2002

# Purpose: to monitor the given file and pop up a text box when the file
# contents change. This shows old contents separated from by a line of hyphens
# from the new contents.


use Tk;
use strict;

# Message from taskbar
my $ReportExceptionPattern = 'FvwmTaskBar: cannot open console';
$ReportExceptionPattern .= '|received SIGPIPE signal: exiting...';

# Message from gtop
$ReportExceptionPattern .= '|glibtop: glibtop_get_proc_time \(\): Client requested field mask';

# Message from fvwm2, mainly for wmconfig
$ReportExceptionPattern .= '|Couldn\'t find pixmap';

# Message from various when killed, e.g. window list module
$ReportExceptionPattern .= '|X connection to :.\.. broken \(explicit kill or server shutdown\)';

# No commas allowed in $SplitLinePattern
my $SplitLinePattern = "bin/favourite-track-exec:";

my $FileName = $ARGV[0];
my $Override = $ARGV[1];
my $ShowAtStart = $ARGV[2];
my $ExcludePrevious = $ARGV[3];
my $ShowDismiss = $ARGV[4];
my $ShowProgressBar = $ARGV[5];
my $ProgressBarMarkerCount = $ARGV[6];

my $WindowBackGround = $ENV{ALU_FMBG_COLOR};
$WindowBackGround = "grey80" if not $WindowBackGround;
my $WindowForeGround = $ENV{ALU_FMFG_COLOR};
$WindowForeGround = "black" if not $WindowForeGround;
my $DisplayBackGround = $ENV{ALU_BG_COLOR};
$DisplayBackGround = "grey20" if not $DisplayBackGround;
my $DisplayForeGround = $ENV{ALU_FG_COLOR};
$DisplayForeGround = "white" if not $DisplayForeGround;

my $ProgressBarValue = 0;

my $Window = MainWindow -> new;

$Window -> title("Monitor of file $FileName");
$Window -> geometry("+0+0");
$Window -> withdraw;
if ($Override)
  { $Window -> overrideredirect(1); }


$Window -> protocol("WM_DELETE_WINDOW",
                          sub { $Window -> withdraw(); });

$Window ->  configure(-background => $WindowBackGround,
                      -foreground => $WindowForeGround);


if ($ShowDismiss)
{
  $Window -> Button(-text => "Dismiss",
                    -relief => 'groove', -borderwidth => 2,
                    -background => $DisplayBackGround,
                    -foreground => $DisplayForeGround,
                    -command => sub { $Window -> withdraw(); }
                   ) -> pack(-side => 'bottom')
             -> focus;
}

if ($ShowProgressBar)
{
  $Window -> Scale(-orient => "horizontal",
                   -relief => 'groove', -borderwidth => 2,
                   -background => $WindowBackGround,
                   -foreground => $WindowForeGround,
                   -troughcolor => $DisplayBackGround,
                   -showvalue => 0,
                   -from => 0, -to => $ProgressBarMarkerCount,
                   -state => "disabled",
                   -variable => \$ProgressBarValue
                  ) -> pack(-side => 'bottom',
                            -fill => 'both', "-expand" => 1);

}

if ($Override)
{
  $Window -> Label(-text => "Monitor of file $FileName",
                   -background => $WindowBackGround,
                   -foreground => $WindowForeGround)
             -> pack(-side => "top");
}

my $Display = $Window -> Scrolled("Text", -scrollbars => 'sw',
                                  -background => $DisplayBackGround,
                                  -foreground => $DisplayForeGround);

$Display -> Subwidget("xscrollbar")
         -> configure(-background => $WindowBackGround);

$Display -> Subwidget("yscrollbar")
         -> configure(-background => $WindowBackGround);

$Display -> configure(-wrap => "none",
                      -state => 'normal');
$Display -> pack(-side => 'top', -fill => 'both', "-expand" => 1);



open FILE, $FileName || die "cannot open $FileName";
my $IncludedPrevious = "";
while (<FILE>)
{
  if (not $ExcludePrevious)
    { $Display -> insert("end", $_); }
  elsif ($_ eq "Start up: Start up\n")
  {
    $IncludedPrevious = "";
    $ProgressBarValue = 0;
  }
  else
  {
    $IncludedPrevious = $IncludedPrevious . $_;
    if ($_ =~ /^Start up: /)
      { $ProgressBarValue++; }
  }
};

if ($ExcludePrevious)
 { $Display -> insert("end", $IncludedPrevious); }

if ($ShowAtStart)
{
  $Display -> see("end");
  $Window -> deiconify();
  $Window -> update();
  $Window -> raise();
}

$Window ->
  repeat(1000,
         sub {
               while (<FILE>)
               {
                 if ($ExcludePrevious && ($_ eq "Start up: Start up\n"))
                 {
                   $Display -> delete("1.0", "end");
                   $ProgressBarValue = 0;
                 }
                 else
                 {
                   if ($_ =~ /^Start up: /)
                     { $ProgressBarValue++; }
                   if (! ($_ =~ /$ReportExceptionPattern/))
                   {
                     if (! $Window -> ismapped)
                     {
                       $Window -> deiconify();
                       $Display -> insert("end", "\n===============================================================================\n");
                     }
                     s,($SplitLinePattern),$1\n ,g;
                     $Display -> insert("end", $_);
                     $Display -> see("end");
                     $Window -> update();
                     $Window -> raise();
                   }
                 } # else
               } # while
             });

MainLoop();

exit;
